
import imaplib

from bzrlib import config, email_message, errors
from bzrlib.mail_client import MailClient


class ImapServerNotConfigured(errors.BzrError):
    _fmt = "No IMAP server hostname configured"


class ImapProtocolError(errors.BzrError):
    _fmt = "IMAP protocol error: %(msg)s"

    def __init__(self, msg):
        self.msg = msg


class ImapCommandError(errors.BzrError):
    _fmt = "IMAP error on %(command)s: %(data)s"

    def __init__(self, command, data):
        self.command = command
        self.data = data


class ImapMailClient(MailClient):
    """A bzrlib mail client that stores the message in an IMAP folder.

    The intended use is to store the messages in the "drafts" folder
    for further editing by the real mail client.
    """

    protocol = 'imap'
    IMAP = imaplib.IMAP4
    default_port = imaplib.IMAP4_PORT

    def __init__(self, config):
        super(ImapMailClient, self).__init__(config)
        self._imap_server = self.config.get_user_option('imap_server')
        if self._imap_server is None:
            raise ImapServerNotConfigured()

        self._imap_port = self.default_port
        colon = self._imap_server.find(':')
        if colon > 0 and self._imap_server.rfind(':') == colon:
            self._imap_port = int(self._imap_server[colon+1:])
            self._imap_server = self._imap_server[:colon]

        self._imap_mailbox = self.config.get_user_option('imap_mailbox')
        self._imap_username = self.config.get_user_option('imap_username')
        self._imap_password = self.config.get_user_option('imap_password')

    def compose(self, prompt, to, subject, attachment,
                mime_subtype, extension, basename=None):
        message = email_message.EmailMessage(
            self.config.username(), to or '', subject or '', '')
        if attachment is not None:
            if basename is None:
                basename = 'attachment'
            message.add_inline_attachment(
                attachment, basename + extension, mime_subtype)

        self.append_message(message.as_string())

    def append_message(self, message):
        auth = config.AuthenticationConfig()
        if self._imap_username is None:
            self._imap_username = auth.get_user(
                self.protocol, self._imap_server, self._imap_port)

        if self._imap_password is None:
            self._imap_password = auth.get_password(
                self.protocol, self._imap_server, self._imap_username,
                self._imap_port)

        try:
            conn = self.IMAP(self._imap_server, self._imap_port)
        except self.IMAP.error, exc:
            raise ImapProtocolError(exc)

        self._check_errors(
            "LOGIN", conn.login, self._imap_username, self._imap_password)
        self._check_errors(
            "APPEND", conn.append, self._imap_mailbox, None, None, message)
        self._check_errors(
            "LOGOUT", conn.logout)

    def _check_errors(self, command, function, *args):
        try:
            type, data = function(*args)
        except self.IMAP.error, exc:
            raise ImapProtocolError(exc)

        if type not in ['OK', 'BYE']:
            raise ImapCommandError(command, data)


class ImapSSLMailClient(ImapMailClient):

    protocol = 'imaps'
    IMAP = imaplib.IMAP4_SSL
    default_port = imaplib.IMAP4_SSL_PORT
